# Copyright (C) 2010 Robert Collins <robertc@robertcollins.net>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. 
#
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0 
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License. 
#
# Part of Tribunal <https://launchpad.net/tribunal>


"""Support for testrepository inside Tribunal.
"""

__all__ = ['GTKUI']

import optparse

import gtk
from testrepository import ui


class TribunalTestRepositoryUI(ui.AbstractUI):
    """A testrepository.ui.AbstractUI that glues it into Tribunal.
    """

    def __init__(self, subunit_window, here):
        """Create a UI to run a TestRepository command under subunit_window.

        :param subunit_window: A SubunitWindow from tribunal-subunit.
        :param here: What should the 'here' be for the UI.
        """
        self.window = subunit_window
        self.here = here

    def _check_cmd(self):
        # TODO: prompt for options and arguments here.
        # Options are available on self.cmd.options
        options = []
        self.options = optparse.Values()
        seen_options = set()
        for option, value in options:
            setattr(self.options, option, value)
            seen_options.add(option)
        if not 'quiet' in seen_options:
            setattr(self.options, 'quiet', False)
        for option in self.cmd.options:
            if not option.dest in seen_options:
                setattr(self.options, option.dest, option.default)
        # Arguments on self.cmd.args
        parsed_args = {}
        unparsed = []
        failed = False
        for arg in self.cmd.args:
            try:
                parsed_args[arg.name] = arg.parse(unparsed)
            except ValueError:
                failed = True
                break
        self.arguments = parsed_args
        return unparsed == [] and not failed

    def output_error(self, error_tuple):
        # Shows the error in a dialog. We could instead have a 'console' on 
        # the SubunitWindow and write the error into the console.
        import traceback
        exctype, value, tb = error_tuple
        as_string = ''.join(traceback.format_exception(exctype, value, tb))
        dialog = gtk.MessageDialog(self.window.subunit_window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            gtk.MESSAGE_ERROR,
            gtk.BUTTONS_CLOSE,
            as_string)
        dialog.run()
        dialog.destroy()

    def output_rest(self, rest_string):
        # TODO: format as HTML, embed a browser?
        raise NotImplementedError(self.output_rest)

    def output_results(self, suite_or_test):
        # TODO: feed to the main window
        raise NotImplementedError(self.output_results)

    def output_stream(self, stream):
        # TODO: ask for a filename to save to.
        raise NotImplementedError(self.output_stream)

    def output_table(self, table):
        raise NotImplementedError(self.output_table)

    def output_values(self, values):
        # TODO: log this to the GUI somehow.
        outputs = []
        for label, value in values:
            outputs.append('%s: %s' % (label, value))
        print '%s\n' % ' '.join(outputs)

    def subprocess_Popen(self, *args, **kwargs):
        # TODO: use a GTK GIO thingy
        import subprocess
        return subprocess.Popen(*args, **kwargs)
